var urlCorrecter = {
	prefs : null,
	prefBranch : "browser.urlbar.",
	
	load : function () {
		this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(this.prefBranch);
		
		this.addTypoFixer();
	},
	
	addTypoFixer : function() {
		if (this.prefs.getBoolPref("offerCorrections")){
		var urlbar = document.getElementById('urlbar');
		
		if (urlbar) {
			urlbar.addEventListener("popupshowing", function (event) { urlCorrecter.addAskFirstOption(event); }, true);
			
			urlbar.oldOnTextEntered = urlbar.onTextEntered;
			
			urlbar.onTextEntered = function (event, justCorrect) {
				// Trim it
				this.value = this.value.replace(/^\s*|\s*$/g,"");
				
				// Determine if the text starts with a keyword
				
				var isShortcut = false;
				
				if (this.value != getShortcutOrURI(this.value, {})){
					isShortcut = true;
				}
				
				if (!isShortcut){
					var res = [
						{find : ";//", replace : "://"},
					
						// Comma or semicolon instead of dot
						{find : "[,;]", replace : "."},
						
						// Incomplete protocol
						{find: "^h?t{1,2}p?(s?)://", "replace" : "http$1://"},
						{find: "^h.{1,3}p(s?)://", "replace" : "http$1://"},
						{find: "^f.{0,2}p://", "replace" : "ftp://"},
						{find: "^(i.{0,2}c|[irc]{2,4})://", "replace" : "irc://"},
						
						// Extra letter before or after protocol
						{find: "^.?(https|http|irc|ftp).?://", "replace" : "$1://"},
						
						// Only two of the three w's
						{find: "^w{2,4}(\\.)", "replace" : "www."},
						{find: "//ww(\\.)", "replace" : "//www."},
						
						// Missing the dot
						{find: "([^\\.])(com|net|org|edu|mil|gov)$", replace : "$1.$2"},
						
						// One letter off/missing one letter
						{nomatch : "(\\.((mo\\.us)|(cc|mm|co|cm|mc|om|mo|co\\.[a-z]{2}))$)", find : "\\.(co[^m]|c[^o]m|[^c]om|[com]{2,3})(\\.[a-z]{2})?$", replace : ".com$2"},
						{nomatch : "(\\.(ee|tt|ne|nt|et|tn)$)", find : "\\.(ne[^t]|n[^e]t|[^n]et|[net]{2,3})(\\.[a-z]{2})?$", replace : ".net$2"},
						{nomatch : "(\\.(gg|ro|gr|orf\\.at)$)", find : "\\.(or[^g]|o[^r]g|[^o]rg|[org]{2,3})(\\.[a-z]{2})?$", replace : ".org$2"},
						{nomatch : "(\\.(ee|eu|de)$)", find : "\\.(ed[^u]|e[^d]u|[^e]du|[edu]{2,3})$", replace : ".edu"},
						
						{nomatch : "(\\.(gg|vg|gv\\.at|vg\\.no|gouv\\.fr)$)", find : "\\.(go[^v]|g[^o]v|[^g]ov|[gov]{2,3})$", replace : ".gov"},
						{nomatch : "(\\.(mm|ml|im|il|li)$)", find : "\\.(mi[^l]|m[^i]l|[^m]il|[mil]{2,3})$", replace : ".mil"},
						
						// Extra letter
						{find : "\\.(c.om|co.m)(\\.[a-z]{2})?$", replace : ".com$2"},
						{find : "\\.(n.et|ne.t)(\\.[a-z]{2})?$", replace : ".net$2"},
						{find : "\\.(o.rg|or.g)(\\.[a-z]{2})?$", replace : ".org$2"},
						{find : "\\.(e.du|ed.u)(\\.[a-z]{2})?$", replace : ".edu$2"},
						{nomatch : "\\.gouv\\.fr$", find : "\\.(g.ov|go.v)(\\.[a-z]{2})?$", replace : ".gov$2"},
						{find : "\\.(m.il|mi.l)(\\.[a-z]{2})?$", replace : ".mil$2"},
						
						// By request - country code corrections
						{find : "\\.([bhj]l|n[k;\\.]|ln)$", replace: ".nl"},
						{find : "\\.(d[234wrsdf]|[wfxc]e)$", replace : ".de"},
						{find : "\\.(n[ip]?)$", replace : ".no"},
						{find : "\\.ku$", replace : ".uk"},
						{find : "\\.ku$", replace : ".uk"},
						
						// Extra letter at the end or before the tld
						{find: "\\..?(com|net|org|edu|mil|gov).?$", replace : ".$1"}
					];
					
					var urlValue;
					
					// Get the domain part
					if (this.value.indexOf("/") == -1){
						// No slashes - only domain
						urlValue = this.value;
					}
					else if (this.value.indexOf("//") == -1){
						// 1+ singles slashes: no protocol, path may be there
						urlValue = this.value.substring(0, this.value.indexOf("/"));
					}
					else if (this.value.indexOf("//") != -1){
						// Protocol is there.
						
						if (this.value.indexOf("/",this.value.indexOf("//") + 2) == -1){
							// only //: protocol and domain
							urlValue = this.value;
						}
						else {
							urlValue = this.value.substring(0, this.value.indexOf("/",this.value.indexOf("//") + 2));
						}
					}
					
					// Save the domain part we found so we can tell if it changed
					var oldValue = urlValue;
					
					for (var i = 0; i < res.length; i++){
						urlValue = urlCorrecter.applyRE(urlValue, res[i]);
					}
					
					if (urlValue != oldValue){
						urlValue = this.value.replace(oldValue, urlValue);
						
						if (urlCorrecter.prefs.getBoolPref("confirmCorrections")){
							var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
							
							var check = {value: false};
							var flags = 
								prompts.BUTTON_TITLE_YES * prompts.BUTTON_POS_0 +
								prompts.BUTTON_TITLE_CANCEL * prompts.BUTTON_POS_1 +
								prompts.BUTTON_TITLE_NO * prompts.BUTTON_POS_2;
							
							var button = prompts.confirmEx(window, "URL Correction", "Did you mean "+urlValue+"?", flags, "Yes", "Cancel", "No", null, check);
							
							if (button == 1){
								return false;
							}
							else if (button == 0){
								this.value = urlValue;
							}
						}
						else {
							this.value = urlValue;
						}
					}
					
					if (justCorrect) {
						return true;
					}
					else {
						return this.oldOnTextEntered();
					}
				}
				else {
					return this.oldOnTextEntered();
				}
			};
					
			// Check for the Go button
			var goButton = document.getElementById("go-button");
			
			if (goButton){
				if (goButton.getAttribute("oncommand")) goButton.setAttribute("oncommand","if (document.getElementById('urlbar').onTextEntered(event, true)) { " + goButton.getAttribute("oncommand") + "}");
				if (goButton.getAttribute("onclick")) goButton.setAttribute("onclick","if (document.getElementById('urlbar').onTextEntered(event, true)) { " + goButton.getAttribute("onclick") + "}");
			}
		}
		}
	},
	
	addAskFirstOption : function (e) {
		var itemID = "url-fixer-askFirst";
		var itemLabel = "Confirm corrections";
		var itemOncommand = "urlCorrecter.toggleAskFirst();";
		
		if ((e.originalTarget.localName == "menupopup") && (!document.getElementById(itemID))){
			var mi = document.createElement("menuitem");
			mi.setAttribute("id", itemID);
			mi.setAttribute("label", itemLabel);
			mi.setAttribute("type","checkbox");
			mi.setAttribute("checked",this.prefs.getBoolPref("confirmCorrections"));
			mi.setAttribute("oncommand", itemOncommand);
			mi.setAttribute("accesskey", "r");
			
			e.originalTarget.appendChild(document.createElement("menuseparator"));
			e.originalTarget.appendChild(mi);
		}
	},
	
	toggleAskFirst : function () {
		this.prefs.setBoolPref("confirmCorrections",!this.prefs.getBoolPref("confirmCorrections"));
		
		if (document.getElementById("url-fixer-askFirst")){
			document.getElementById("url-fixer-askFirst").setAttribute("checked",this.prefs.getBoolPref("confirmCorrections"));
		}
	},
	
	applyRE : function (string, re){
		if (!re.nomatch || !string.match(new RegExp(re.nomatch))){
			if (re.res){
				for (var i = 0; i < re.res.length; i++){
					string = this.applyRE(string, re.res[i]);
				}
			}
			else {
				string = string.replace(new RegExp(re.find, "gi"), re.replace);
			}
		}
		
		return string;
	}
};
